var StoreLocator = function(config) {
	// Save Reference to Script Name
	this.name = "Store Locator";
    // Save Reference to Script Name
    this.scriptName = "Store_Locator";
    // Script Version
    var scriptVersion = "6.0.0";
    // Last Modified
    var lastModified = "2017-08-30";
    // Last Modified
    var lastUploaded = "2017-08-30";
    // Block Id
    this.blockId = 6145;
    // Save Reference to Config Object
    this.config = config;
    // status = (0: init, 1: waiting for search, 2: data loaded, 3: map rendering, 4: map shown, 5: map hidden, 8: multiple matches found, 9: no matches found)
    this.STATUS_CODES = {
        INIT: 0,
        WAITING_FOR_SEARCH: 1,
        DATA_LOADED: 2,
        MAP_RENDERING: 3,
        MAP_SHOWN: 4,
        MAP_HIDDEN: 5,
        MULTIPLE_MATCHES_FOUND: 8,
        NO_MATCHES_FOUND: 9
    };
    // Place callback on window for JSONP
    window.parseResponse = this.parseResponse;
    // Establish Boolean for tracking
    this.trackOnce = true;
	// Establish Reference for timeout
	window.searchTimeout;
    // Init Class
    this.init(arguments[1]);
    // Establish Reference to this
    var self = this;
    // Call Init Method once AdKit is loaded
    adkit.onReady(function() {
	    self.ebReady();
	});
};

StoreLocator.prototype = {
    // Function for getting elements
    d: function(id) { return document.getElementById(id); },
    // Function for getting unique id
    genUid: function() { return "ui_" + Math.ceil((new Date()).valueOf() * Math.random()).toString().substr(0, 7); },
    // Organize Options from Config object
    fetchOptions: function() {
        for (var i = 0; i < this.availableOptions.length; i++) {
            this[this.availableOptions[i].name] = this.availableOptions[i].value;
            if (this.config.hasOwnProperty("options")) {
                if (this.config.options.hasOwnProperty(this.availableOptions[i].name)) {
                    this[this.availableOptions[i].name] = this.config.options[this.availableOptions[i].name];
                }
            }
        }
    },
    fetchTemplates: function() {
        for (var i = 0; i < this.availableTemplates.length; i++) {
            this[this.availableTemplates[i].name + "Template"] = this.availableTemplates[i].value;
            if (this.config.hasOwnProperty("templates")) {
                if (this.config.templates.hasOwnProperty(this.availableTemplates[i].name)) {
                    this[this.availableTemplates[i].name + "Template"] = this.config.templates[this.availableTemplates[i].name];
                }
            }
        }
    },
    fetchCustomCSS: function() {
        var i;
        for (i = 0; i < this.availableCSS.length; i++) {
            this[this.availableCSS[i] + "CSS"] = this.cssPrefix + " " + this.availableCSS[i];
            this.availableCSS[i] = "." + this.cssPrefix + "." + this.availableCSS[i];
        }
        if (this.config.hasOwnProperty("customCSS")) {
            for (i = 0; i < this.config.customCSS.length; i++) {
                var defaultName = this.config.customCSS[i].split("=")[0].replace(/^\s*/, "").replace(/\s*$/, "");
                var newName = this.config.customCSS[i].split("=")[1].replace(/^\s*/, "").replace(/\s*$/, "");
                defaultName = defaultName.replace(this.cssPrefix, "").replace(/\./g, "");
                newName = newName.substr(0, 1) == "." ? newName.substr(1, newName.length) : newName;
                this[defaultName + "CSS"] = newName.replace(/\./g, " ");
            }
        }
    },
    bindToEB: function() {
        if (typeof EB == "undefined") {
            EB = {};
        }
        if (EB) {
            if (!EB.hasOwnProperty("_block")) {
                EB._block = [];
            }
            EB._block[EB._block.length] = this;
        }
    },
    extend: function(destination, source, callback, param) {
        for (var property in source) {
            if (!source.hasOwnProperty(property)) continue;
            destination[property] = source[property];
        }
        if (callback) {
            var _param = param ? "'" + param + "'" : "";
            try {
                eval(callback + "(" + _param + ")");
            } catch (err) {}
        }
    },
    appendJSTag: function(src, id) {
        var jsTag = document.createElement("script");
        if (id) {
            jsTag.id = id;
        }
        jsTag.src = src;
        document.getElementsByTagName("head")[0].appendChild(jsTag);
        return jsTag;
    },
    getTargetObject: function(prop, find) {
        for (var i = 0; i < EB._block.length; i++) {
            if (EB._block[i][prop] == find) {
                return EB._block[i];
            }
        }

        return null;
    },
    getObjectId: function(obj) {
        if (typeof obj.id != "undefined") {
            return obj.id;
        }
        return null;
    },
    mapModuleLoaded: function(param) {
        for (var i = 0; i < EB._block.length; i++) {
            if (EB._block[i].displayMap == param) {
                if (!EB._block[i].proceeded) {
                    EB._block[i].buildMapStage();
                }
            }
        }
    },
    init: function() {
	    // Establish Scope
        var _self = this;
        // Create Unique Id
        this.uid = this.genUid();
        this.container = this.d(this.config.container);
        this.availableOptions = [{
            name: "eventNamePrefix",
            value: "Store_Locator"
        }, {
            name: "maxMatches",
            value: 3
        }, {
            name: "countryCode",
            value: ""
        }, {
            name: "radius",
            value: 20
        }, {
            name: "initialZoom",
            value: 10
        }, {
            name: "panDistance",
            value: 100
        }, {
            name: "numberedMarker",
            value: false
        }, {
            name: "markerIcon",
            value: "https://www.mapquestapi.com/staticmap/geticon?uri=poi-{n}.png"
        }, {
            name: "googleMapKey",
            value: ""
        }, {
            name: "MapQuestApiKey",
            value: ""
        }, {
            name: "bingMapKey",
            value: ""
        }, {
            name: "displayMap",
            value: "mapquest"
        }, {
            name: "directionsService",
            value: "mapquest"
        }, {
            name: "closeControl",
            value: false
        }, {
            name: "panControl",
            value: false
        }, {
            name: "zoomControl",
            value: false
        }, {
            name: "zoomControlPosition",
            value: "TL"
        }, {
            name: "customZoomControl",
            value: false
        }, {
            name: "mouseHoverInfo",
            value: false
        }, {
            name: "customScrollbar",
            value: false
        }, {
            name: "scrollbarWidth",
            value: 12
        }, {
            name: "scrollbarTrackColor",
            value: "lightgray"
        }, {
            name: "scrollbarHandlerColor",
            value: "#343434"
        }, {
            name: "searchEngagedCallback",
            value: ""
        }, {
            name: "showMapCallback",
            value: ""
        }, {
            name: "collapseMapCallback",
            value: ""
        }, {
            name: "showMessageCallback",
            value: ""
        }, {
            name: "collapseMessageCallback",
            value: ""
        }, {
            name: "zipCodeValidation",
            value: false
        }];
        this.availableTemplates = [{
            name: "infoWindow",
            value: "<b>{STORE_NAME}</b><br/>{ADDRESS}<br/>{CITY}, {STATE} {ZIPCODE}<br/>{DIRECTIONS}"
        }, {
            name: "getDirections",
            value: "Get Directions"
        }, {
            name: "noMatchesFoundMessage",
            value: "Sorry, no matches found"
        }, {
            name: "multiMatchesFoundMessage",
            value: "Multiple matches found, please refine your search"
        }, {
            name: "userLocationFailedMessage",
            value: "Sorry, share location denied by user"
        }, {
            name: "userLocationTimedOutMessage",
            value: "Sorry, request has timed out"
        }, {
            name: "invalidZipCodeMessage",
            value: "Please input valid Zip Code"
        }, {
            name: "tryAgain",
            value: "Try Again"
        }, {
            name: "closeMessage",
            value: "Ok"
        }];
        this.cssPrefix = "storeLocator";
        this.availableCSS = ["mapStage", "mapLoader", "resultContainer", "infoWindow", "collapseMapButton", "zoomInButton", "zoomOutButton", "panEastButton", "panWestButton", "panSouthButton", "panNorthButton", "messageWindow", "userLocationFailed", "closeMessage", "multiMatchesFound", "noMatchesFound", "invalidZipCode", "tryAgain"];
        this.config.actionRegister = undefined;

        this.fetchOptions();
        this.fetchTemplates();
        this.fetchCustomCSS();

        // Check if width and height are set to zero, and go full screen
        this.width = this.config.width;
        this.height = this.config.height;
        // Check if width and height are set to zero, and go full screen
        if (this.width === 0 && this.height === 0) {
            // Get Parent Element Width
            this.width = "100%";
            // Get Parent Element Height
            this.height = this.container.parentNode.clientHeight;
            // add Event listener to resize map
            window.addEventListener("resize", function() {
				// Update Width of map container
				_self.container.style.width = "100%";
		        // Update Height of map container
		        _self.container.style.height = _self.container.parentNode.clientHeight + "px";
	        }, false);
        }
        // Save Reference to Gizmo Table Id
        this.hostedData = this.config.hostedDataTable;
        // Determine whether site is using Secure Protocol
        this.proto = ('https:' == document.location.protocol ? 'https://' : 'http://');
        // Build Gizmo API
        this.dataUrl = this.proto + "gizmo.serving-sys.com/api/data/get/" + this.hostedData;
        this.mapModules = ["none", "mapquest", "leaflet", "google", "bing"];
        this.initialZoom = this.initialZoom > 18 ? 18 : this.initialZoom < 1 ? 1 : this.initialZoom;
        this.customMarker = false;
        this.container.style.width = this.width + "px";
        this.container.style.height = this.height + "px";
        this.container.className = this.mapStageCSS;
        this.container.style.zIndex = 0;
        this.container.style.display = "block";
        this.status = this.STATUS_CODES.INIT;
        this.apiLoaderTimer = null;

        // If Country Code is not set, use API to determine
        if (this.countryCode === "") {
        	try { this.countryCode = EB.API.Ad.getAdData("geoData").country + '-'; } catch (err) {}
        } else if (this.countryCode !== "") {
        	this.countryCode = this.countryCode + '-';
        }

        this.bindToEB();

        this.ua = navigator.userAgent.toLowerCase();
        //document.getElementById("zipcode").value = 'test 1';
        this.isiOS = this.ua.match(/ipod|iphone|ipad/) !== null;
        this.isAndroid = this.ua.match(/android/) !== null;
        this.isAndroid2 = this.ua.match(/android 2/) !== null;
        this.isKindleFire = this.ua.match(/android|silk/) !== null && this.ua.match(/kf\w+\sbuild/) !== null;
        if (this.isiOS || this.isAndroid) {
            this.mouseHoverInfo = false;
        }

        this.useMap = false;
        this.displayMap = this.displayMap.toLowerCase();
        this.displayMap = this.displayMap == "none" ? "list" : this.displayMap;
        for (var i = 0; i < this.mapModules.length; i++) {
            if (this.displayMap == this.mapModules[i]) {
                this[this.mapModules[i].toUpperCase() + "MAP"] = true;
                if (this.displayMap != "list") {
                    this.useMap = true;
                }
            }
        }
        if (this.displayMap != "list" && this.displayMap != "leaflet") {
            this.directionsService = this.displayMap;
        }
        if (this.config.hasOwnProperty("options")) {
            if (this.config.options.hasOwnProperty("markerIcon")) {
                this.customMarker = true;
            }
        }
        switch (this.zoomControlPosition) {
            case "TR":
                this.zoomControlPosition = "TOP_RIGHT";
                break;
            case "BR":
                this.zoomControlPosition = "BOTTOM_RIGHT";
                break;
            case "BL":
                this.zoomControlPosition = "BOTTOM_LEFT";
                break;
            default:
                this.zoomControlPosition = "TOP_LEFT";
        }
        if (this.isAndroid) {
            this.container.style.webkitTapHighlightColor = "rgba(0, 0, 0, 0)";
        }
    },

    ebReady: function() {
        if (EB._block) {
            for (var i = 0; i < EB._block.length; i++) {
                if (EB._block[i].name == "Store Locator") {
                    if (typeof EB._block[i][EB._block[i].displayMap + "ModuleLoaded"] == "undefined") {
                        var moduleTag = document.createElement("script");
                        moduleTag.src = "store_locator/modules/" + EB._block[i].displayMap + "Module.js";
                        document.body.appendChild(moduleTag);
                    } else {
                        EB._block[i].buildMapStage();
                    }
                }
            }
        }

        var self = this;

        // Listen for EB Messages
        window.addEventListener("message", function(event) {
            // Establish Object
            var obj;
            // Evaluate JSON first before proceeding. When the jsonstring can't be evaluated, do not proceed.
            try {
                obj = eval(JSON.parse(event.data));
            } catch (e) {
                return;
            }
            // Handle Messages
            self.handleEventData(obj);
        });

        // Send Message
        EB._sendMessage('getPandoraStatus', {});
    },

    // Handle Messages from Listener
    handleEventData: function(_obj) {
        // Listen for Custom Script for Connection Message
        if (_obj.type === 'pandoraStatus') {
            window.isInPandora = true;
        }
    },

    buildMapStage: function() {
        this.proceeded = true;
        var _div = document.createElement("div");
        _div.className = this.resultContainerCSS;
        _div.id = this.uid + "ResultContainer";
        _div.style.position = "relative";
		if (this.displayMap == 'mapquest') {
			_div.style.width = this.width + "px";
	        _div.style.height = this.height + "px";
		} else {
			_div.style.width = "100%";
	        _div.style.height = "100%";
		}
        _div.style.overflow = "hidden";
        this.container.appendChild(_div);
        this.resultContainer = _div;

        _div = document.createElement("div");
        _div.className = this.mapLoaderCSS;
        _div.style.display = "none";
        this.resultContainer.appendChild(_div);
        this.mapLoaderScreen = _div;

        _div = document.createElement("div");
        _div.className = this.messageWindowCSS;
        _div.style.display = "none";
        this.resultContainer.appendChild(_div);
        this.messageWindow = _div;

        if (this.useMap) {
            _div = document.createElement("div");
            _div.id = this.uid + "MapContainer";
            _div.style.position = "relative";
			if (this.displayMap == 'mapquest') {
				_div.style.width = this.width + "px";
		        _div.style.height = this.height + "px";
			} else {
				_div.style.width = "100%";
		        _div.style.height = "100%";
			}
            this.resultContainer.appendChild(_div);
            this.mapContainer = _div;

            var panBtns = ["East", "West", "South", "North"];
            var i;
            for (i = 0; i < panBtns.length; i++) {
                _div = document.createElement("div");
                _div.id = this.uid + "Pan" + panBtns[i] + "Button";
                _div.className = this["pan" + panBtns[i] + "ButtonCSS"];
                _div.style.display = "none";
                this.resultContainer.appendChild(_div);
                _div.addEventListener("click", StoreLocator.prototype["pan" + panBtns[i]], false);
                this["pan" + panBtns[i] + "Btn"] = _div;
            }
            var zoomBtns = ["In", "Out"];
            for (i = 0; i < zoomBtns.length; i++) {
                _div = document.createElement("div");
                _div.id = this.uid + "Zoom" + zoomBtns[i] + "Button";
                _div.className = this["zoom" + zoomBtns[i] + "ButtonCSS"];
                _div.style.display = "none";
                this.resultContainer.appendChild(_div);
                _div.addEventListener("click", StoreLocator.prototype["zoom" + zoomBtns[i]], false);
                this["zoom" + zoomBtns[i] + "Btn"] = _div;
            }
        }

        _div = document.createElement("div");
        _div.id = this.uid + "CollapseMapButton";
        _div.className = this.collapseMapButtonCSS;
        _div.style.display = "none";
        this.resultContainer.appendChild(_div);
        _div.addEventListener("click", StoreLocator.prototype.collapseMap, false);
        this.closeMapBtn = _div;

        this.status = this.STATUS_CODES.WAITING_FOR_SEARCH;
    },

    clickToGetDirections: function(poi, uid) {
        var slObj = this;
        if (uid) {
            slObj = StoreLocator.prototype.getTargetObject("uid", uid);
        }
        // Save Point of interest Data
        var data = slObj.responseData["Store List"][poi];
        // Build Address from Response data
        var address = data["Address"] + " " + data["City"] + " " + data["State"] + " " + data["Zip Code"];
        var directionLink = slObj[slObj.directionsService + "DirectionLink"](address, data);
        // for mobile devices...
        if (slObj.isKindleFire) {
            directionLink = "https://maps.google.com/?daddr=" + encodeURI(address);
        } else if (slObj.isAndroid) {
            directionLink = "geo:0,0?q=" + encodeURI(address);
        } else if (slObj.displayMap == 'mapquest' && slObj.isiOS) {
            directionLink = "https://www.mapquest.com/?le=t&q=" + encodeURI(address) + "&maptype=map&vs=map";
        } else if (slObj.isiOS) {
            directionLink = "https://maps.google.com/?daddr=" + encodeURI(address);
        }
        return directionLink;
    },

    launchDirectionWindow: function(_e) {
        EB.userActionCounter(_e + "_GetDirectionsClicked");
    },

    renderInfoWindow: function(data, poi) {
        // Establish Reference for Get Directions Link
        var directionTag;
        // Check if in Pandora
        if (window.isInPandora) {
            directionTag = "<a href=\"javascript:;\" on" + (this.isiOS || this.isAndroid ? "TouchStart" : "Click") + "=\"top.location = '" + this.clickToGetDirections(poi, this.uid) + "'" + ";EB." + "userActionCounter('" + this.eventNamePrefix + "_GetDirectionsClicked');\">";
        } else if (this.displayMap == 'mapquest' && this.isiOS) {
            directionTag = "<a href=\"javascript:;\"; ontouchstart=\"top.location='" + this.clickToGetDirections(poi, this.uid) + "'; javascript:StoreLocator.prototype.launchDirectionWindow('" + this.eventNamePrefix + "');\">";
        } else {
            directionTag = "<a href=\"" + this.clickToGetDirections(poi, this.uid) + "\" target=\"_blank\"; onclick=\"javascript:StoreLocator.prototype.launchDirectionWindow('" + this.eventNamePrefix + "');\">";
        }
        directionTag += this.getDirectionsTemplate + "</a>";

        // Verify Response object has all necessary data for info window
        var storeName = data["Store"] || '';
        var address = data["Address"] || '';
        var city = data["City"] || '';
        var state = data["State"] || '';
        var zipCode = data["Zip Code"] || '';
		// Return Info Window html
        return "<div class=\"" + this.infoWindowCSS + "\">" + this.infoWindowTemplate.replace(/{STORE_NAME}/g, storeName).replace(/{ADDRESS}/g, address).replace(/{CITY}/g, city).replace(/{STATE}/g, state).replace(/{ZIPCODE}/g, zipCode).replace(/{DIRECTIONS}/g, directionTag) + "</div>";
    },

    displayResults: function() {
        this.poi = [];
        this["display" + this.displayMap.substr(0, 1).toUpperCase() + this.displayMap.substr(1, this.displayMap.length) + "Results"]();
    },
    hideMessage: function() {
        var uid = null;
        var slObj = this;
        if (typeof this.id != "undefined") {
            uid = this.id;
        }
        if (typeof arguments[0] == "string") {
            uid = arguments[0];
        }
        if (uid) {
            slObj = StoreLocator.prototype.getTargetObject("uid", uid.split("CollapseMap")[0]);
        }
        slObj.messageWindow.style.display = "none";
		slObj.status = slObj.STATUS_CODES.WAITING_FOR_SEARCH;
        if (slObj.collapseMessageCallback !== "") {
            if (typeof slObj.collapseMessageCallback == "function") {
                try {
                    eval(slObj.uid + "SLExecCallback = " + slObj.collapseMessageCallback);
                } catch (err) {}
                try {
                    eval(slObj.uid + "SLExecCallback()");
                } catch (err) {}
            }
        }
    },
    displayMessage: function(msg) {
		// Clear Message Window
		this.messageWindow.innerHTML = "";
		// Hide Close button is showing
		try { this.d(this.uid + "CollapseMapButton").style.display = "none"; } catch (err) {}
		// Check Message
        if (msg == "multiMatchesFound") {
			this.messageWindow.innerHTML = "<div class=\"" + this.multiMatchesFoundCSS + "\">" + this.multiMatchesFoundMessageTemplate + "</div><div class=\"" + this.tryAgainCSS + "\" id=\"" + this.uid + "CollapseMap\" on" + (this.isiOS ? 'touchend' : 'Click') + "=\"StoreLocator.prototype.hideMessage('" + this.uid + "CollapseMap')\">" + this.tryAgainTemplate + " </div>";
        } else if (msg == "noMatchesFound") {
			this.messageWindow.innerHTML = "<div class=\"" + this.noMatchesFoundCSS + "\">" + this.noMatchesFoundMessageTemplate + "</div><div class=\"" + this.tryAgainCSS + "\" id=\"" + this.uid + "CollapseMap\" on" + (this.isiOS ? 'touchend' : 'Click') + "=\"StoreLocator.prototype.hideMessage('" + this.uid + "CollapseMap')\">" + this.tryAgainTemplate + " </div>";
        } else if (msg == "userLocationFailed") {
			this.messageWindow.innerHTML = "<div class=\"" + this.userLocationFailedCSS + "\">" + this.userLocationFailedMessageTemplate + "</div><div class=\"" + this.closeMessageCSS + "\" id=\"" + this.uid + "CollapseMap\" on" + (this.isiOS ? 'touchend' : 'Click') + "=\"StoreLocator.prototype.hideMessage('" + this.uid + "CollapseMap')\">" + this.closeMessageTemplate + " </div>";
        } else if (msg == "userLocationTimedOut") {
			this.messageWindow.innerHTML = "<div class=\"" + this.userLocationFailedCSS + "\">" + this.userLocationTimedOutMessageTemplate + "</div><div class=\"" + this.closeMessageCSS + "\" id=\"" + this.uid + "CollapseMap\" on" + (this.isiOS ? 'touchend' : 'Click') + "=\"StoreLocator.prototype.hideMessage('" + this.uid + "CollapseMap')\">" + this.closeMessageTemplate + " </div>";
        } else if (msg == "invalidZipCode") {
			this.messageWindow.innerHTML = "<div class=\"" + this.invalidZipCodeCSS + "\">" + this.invalidZipCodeMessageTemplate + "</div><div class=\"" + this.tryAgainCSS + "\" id=\"" + this.uid + "CollapseMap\" on" + (this.isiOS ? 'touchend' : 'Click') + "=\"StoreLocator.prototype.hideMessage('" + this.uid + "CollapseMap')\">" + this.tryAgainTemplate + " </div>";
        }
        if (this.messageWindow.innerHTML !== "") {
            this.mapLoaderScreen.style.display = "none";
            if (this.useMap) {
                this.mapContainer.style.opacity = 0;
            }

			this.container.style.display = this.messageWindow.style.display = "block";

			if (this.showMessageCallback !== "") {
                if (typeof this.showMessageCallback == "function") {
                    try {
                        eval(this.uid + "SLExecCallback = " + this.showMessageCallback);
                    } catch (err) {}
                    try {
                        eval(this.uid + "SLExecCallback()");
                    } catch (err) {}
                }
            }

			this.messageWindow.focus();
        }
    },
    parseResponse: function(response) {
		// Clear Timeout
		clearTimeout(window.searchTimeout);
		// hide any message that might still be showing
		try { this.hideMessage(); } catch (err) {}
        // Clear out response object
        var slObj = null;
        // Check for valid response
        if (response.hasOwnProperty("Store List")) {
            // Build Response
            slObj = StoreLocator.prototype.getTargetObject();
            // slObj = StoreLocator.prototype.getTargetObject("hostedData", response.hostedData[0].tableName);
        }
        // Check for Store Matches
        if (response["Store List"].length < 1) {
            // Display No response
            slObj.displayMessage("noMatchesFound");
            // Update Status - add constants
            slObj.status = slObj.STATUS_CODES.NO_MATCHES_FOUND;
        } else {
            // Check if object exists
            if (slObj !== null) {
                // Update Object with data from Gizmo
                slObj.responseData = response;
                // Remove Source Tag
                document.getElementsByTagName("head")[0].removeChild(slObj.dataSourceTag);
            }
            // Update Status
            slObj.status = slObj.STATUS_CODES.DATA_LOADED;
            // Show Results
            slObj.displayResults();
        }
    },
    showCloseControl: function() {
        this.closeMapBtn.style.display = "block";
    },
    hideCloseControl: function() {
        this.closeMapBtn.style.display = "none";
    },
    showCustomZoomControl: function() {
        this.zoomInBtn.style.display = this.zoomOutBtn.style.display = "block";
    },
    hideCustomZoomControl: function() {
        this.zoomInBtn.style.display = this.zoomOutBtn.style.display = "none";
    },
    showPanControl: function() {
        this.panEastBtn.style.display = this.panWestBtn.style.display = this.panSouthBtn.style.display = this.panNorthBtn.style.display = "block";
    },
    hidePanControl: function() {
        this.panEastBtn.style.display = this.panWestBtn.style.display = this.panSouthBtn.style.display = this.panNorthBtn.style.display = "none";
    },
    showMap: function() {
        this.mapLoaderScreen.style.display = "none";
        if (this.useMap) {
            this.mapContainer.style.opacity = 1;
        }
        this.container.style.display = "block";
        if (this.closeControl) {
            this.showCloseControl();
        }
        if (this.useMap && this.panControl) {
            this.showPanControl();
        }
        if (this.useMap && this.customZoomControl) {
            this.showCustomZoomControl();
        }
        this.container.focus();
        if (this.showMapCallback !== "") {
            if (typeof this.showMapCallback == "function") {
                try {
                    eval(this.uid + "SLExecCallback = " + this.showMapCallback);
                } catch (err) {}
                try {
                    eval(this.uid + "SLExecCallback()");
                } catch (err) {}
            }
        }
        this.status = this.STATUS_CODES.MAP_SHOWN;
    },
    hideMap: function() {
        this.container.style.display = this.closeMapBtn.style.display = "none";
        this.hideMessage();
        if (this.useMap) {
            this.hidePanControl();
            this.hideCustomZoomControl();
        }
        this.status = this.STATUS_CODES.MAP_HIDDEN;
    },
    collapseMap: function() {
        var uid = null;
        var slObj = this;
        if (typeof this.id != "undefined") {
            uid = this.id;
        }
        if (typeof arguments[0] == "string") {
            uid = arguments[0];
        }
        if (uid) {
            slObj = StoreLocator.prototype.getTargetObject("uid", uid.split("CollapseMap")[0]);
        }
        if (slObj.hasOwnProperty("container")) {
            slObj[slObj.displayMap + "CleanUp"]();
            slObj.hideMap();
            slObj.hideMessage();
            if (slObj.collapseMapCallback !== "") {
                if (typeof slObj.collapseMapCallback == "function") {
                    try {
                        eval(slObj.uid + "SLExecCallback = " + slObj.collapseMapCallback);
                    } catch (err) {}
                    try {
                        eval(slObj.uid + "SLExecCallback()");
                    } catch (err) {}
                }
            }
        }
    },

	// Handle Search
	search: function(sStr) {
		// Establish Scope
		var _self = this;
		// Hide Message if it's showing
	    this.hideMessage();
		// Handle Feature Tracking
        this.trackAdFeatures('data_impression', 'AF_' + this.scriptName + '_' + this.blockId);

		// Stop if there is nothing in text field
		if (!sStr) { return false; }
        sStr = sStr.replace(/^\s*/, "").replace(/\s*$/, "");
        this.container.style.opacity = 1;
        this.container.style.display = "block";
        this.mapLoaderScreen.style.display = "block";
        if (this.searchEngagedCallback !== "") {
            if (typeof this.searchEngagedCallback == "function") {
                try {
                    eval(this.uid + "SLExecCallback = " + this.searchEngagedCallback);
                } catch (err) {}
                try {
                    eval(this.uid + "SLExecCallback()");
                } catch (err) {}
            }
        }

        if (sStr == "location" && !arguments[1]) {
            this.getUserLocation();
        } else {
            if (sStr != "location" && this.zipCodeValidation) {
                if (sStr.match(/(^\d{5}$)|(^\d{5}-\d{4}$)/) === null) {
                    this.displayMessage("invalidZipCode");
                    return false;
                }
            }

            this.origin = sStr == "location" && arguments[1] ? arguments[1] : sStr;

            // Verify Origin
            if (this.origin !== "") {
                // Check if UK Postal Code
                var postalCode = this.validate_postcode(this.origin);
                // Clean up string
                this.origin = this.origin.replace(/^\s*/, "").replace(/\s*$/, "");
                // Check if User Input Zip Code
                if (postalCode) {
                    // Build full API url for Postal Code Search
                    this.dataSourceTag = this.appendJSTag(this.dataUrl + "?maxmatches=" + this.maxMatches + "&prox=" + this.radius + "&zip=" + this.countryCode + this.origin);
                    // Handle Tracking
                    EB.userActionCounter(this.eventNamePrefix + "_SearchByUserInput");
                } else if (sStr == "location") {
                    // Build full API url for User Location Search
                    this.dataSourceTag = this.appendJSTag(this.dataUrl + "?fset=3&" + this.origin + "&maxmatches=" + this.maxMatches + "&prox=" + this.radius + "&zip=" + this.countryCode + EB.API.Ad.getAdData("geoData").zip);
                    // Handle Tracking
                    EB.userActionCounter(this.eventNamePrefix + "_SearchByUser" + (sStr == "location" ? "Location" : "Input"));
                } else {
                    // Build full API url for City State Search
                    this.dataSourceTag = this.appendJSTag(this.dataUrl + "?fset=5&lat=" + EB.API.Ad.getAdData("geoData").latitude + "&long=" + EB.API.Ad.getAdData("geoData").longitude + "&maxmatches=" + this.maxMatches + "&q=" + encodeURIComponent(this.origin) );
                    // Handle Tracking
                    EB.userActionCounter(this.eventNamePrefix + "_SearchByUserInput");
                }
				// Clear Timout
		        clearTimeout(window.searchTimeout);
				// Set Timeout for error handling
                window.searchTimeout = setTimeout(function() { _self.displayMessage("userLocationTimedOut"); }, 15000);
            }
        }
    },
    getUserLocation: function() {
        this.search("location", "lat=" + EB.API.Ad.getAdData("geoData").latitude + "&long=" + EB.API.Ad.getAdData("geoData").longitude);
    },
    getStatus: function() {
        // status = (0: init, 1: waiting for search, 2: data loaded, 3: map rendering, 4: map shown, 5: map hidden, 8: multiple matches found, 9: no matches found)
        return this.status;
    },

	// Handle Postal Code Validation - Update with each new market
	validate_postcode: function(_postcode) {
		// Establish Boolean for tracking valid zip code
		var isValid = false;
		// Check which Country we're searching in
		switch(this.countryCode.toString()) {

			case "840-": // US
				// Validate US Postal Codes
				isValid = /^\d{5}(-\d{4})?$/.test(_postcode);
				break;

			case "826-": // UK
				// Break Postal Code into outer and inner codes
				_postcode = _postcode.split(' ');
				_postcode = _postcode[0].replace(/\s/g, "");
				// Define expression
				var regex = /^(GIR 0AA|[A-PR-UWYZ]([A-HK-Y]([0-9][A-Z]?|[1-9][0-9])|[1-9]([0-9]|[A-HJKPSTUW])?))$/i;
				// Validate UK Postal Codes
				isValid = regex.test(_postcode);
				if (isValid) { this.origin = _postcode; }
				break;

			case "124-": // Canada
				// Validate Canadian Postal Codes
				isValid = /^[A-Za-z]\d[A-Za-z][ -]?\d[A-Za-z]\d$/.test(_postcode);
				// Remove spaces
				if (isValid) { this.origin = _postcode.replace(' ', ''); }
				break;

			case "276-": // Germany
				// Validate German Postal Codes
				isValid = /^(?!01000|99999)(0[1-9]\d{3}|[1-9]\d{4})$/.test(_postcode);
				break;
			default: // Default to US
				isValid = /^\d{5}(-\d{4})?$/.test(_postcode);
		}

		return isValid;
	},

	// DO NOT REMOVE THE FOLLOWING
    // Track Ad Features - accepts interaction name and impression type
    trackAdFeatures: function(_interaction, _noun) {
        // Establish var to track local mode
        var isServed;
        // Verify ad is not local
        try {
            isServed = EB._isServingMode();
        } catch (err) {}

        if (isServed && this.trackOnce) {
            try {
                // Update Boolean
                this.trackOnce = false;
                // Grab Ad ID
                var adId = EB._adConfig.adId;
                // Grab Session ID
                var sId = EB._adConfig.sID;
                // Build Tracking Pixel
                var trackingPixel = "https://bs.serving-sys.com/BurstingPipe/adServer.bs?cn=tf&c=19&mc=imp&pli=16479316&PluID=0&ord=%time%&rtu=-1&pcp=$$sID=" + sId + "|adID=" + adId + "|interactionName=" + _interaction + "|noun=" + _noun + "$$";
                // Fire Tracking Pixel by creating a new image.
                new Image().src = trackingPixel;

            } catch (err) {}
        }
    }
};
