StoreLocator.prototype.extend(StoreLocator.prototype, {
	displayLeafletResults: function(){
		if(typeof L == "undefined"){
			this.loadLeafletAPI();
		}else{
			this.renderLeafletMap();
		}
	},
	leafletAPICallback: function(){
		for(var i = 0; i < EB._block.length; i ++){
			if(EB._block[i].LEAFLETMAP && EB._block[i].status == 2){
				clearInterval(EB._block[i].apiLoaderTimer);
				EB._block[i].renderLeafletMap();
			}
		}
	},
	loadLeafletAPI: function(){
		if(!this.d("leafletAPICSS")){
			var linkTag = document.createElement("link");
			linkTag.id = "leafletAPICSS";
			linkTag.rel = "stylesheet";
			linkTag.href = "https://cdnjs.cloudflare.com/ajax/libs/leaflet/0.7.7/leaflet.css";
			document.getElementsByTagName("head")[0].appendChild(linkTag);
		}
		if(!this.d("leafletAPIJS")){ this.appendJSTag("https://cdnjs.cloudflare.com/ajax/libs/leaflet/0.7.7/leaflet.js", "leafletAPIJS"); }
		this.apiLoaderTimer = setInterval(function(){
			if(typeof L != "undefined"){ StoreLocator.prototype.leafletAPICallback(); }
		}, 50);
	},
	renderLeafletMap: function(){
		this.status = 3;
		this.mapContainer.style.display = "block";
		var centerSet = null;

		// Loop through Response Data
		for(var i = 0; i < this.responseData["Store List"].length; i++){
			// Save Reference to Store Location Data
			var data = this.responseData["Store List"][i];

			if(!this.MAP){
				this.MAP = L.map(this.mapContainer.id, { zoomControl: false, keyboard: false });
				// Add Fix for IE
				L.Map.addInitHook(function () { return L.DomEvent.off(this._container, "mousedown", this.keyboard._onMouseDown); });
				if(this.zoomControl && !this.customZoomControl){
					var zoomCtrl = new L.Control.Zoom({position: this.zoomControlPosition.toLowerCase().replace("_", "")});
					zoomCtrl.addTo(this.MAP);
				}
				L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
					attribution: 'Map data &copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a>',
					maxZoom: 18
				}).addTo(this.MAP);
			}

			// Save Center Point
			if(centerSet == null){ centerSet = this.MAP.setView([data["+latitude"], data["+longitude"]], this.initialZoom, true); }
			var infoWindow = this.renderInfoWindow(data, i);
			var markerCofig = {};
			if(this.numberedMarker || this.customMarker){ markerCofig.icon = L.icon({ iconUrl : this.markerIcon.replace("{n}", (i + 1)) }); }
			// Save Marker Location
			this.poi[i] = L.marker([data["+latitude"], data["+longitude"]], markerCofig).addTo(this.MAP).bindPopup(infoWindow);
			this.poi[i].slMapObj = this;
			this.poi[i].on("click", function(){
				EB.userActionCounter(this.slMapObj.eventNamePrefix + "_LocationMarkerClicked");
			});
		}
		// simulate drag on map to make sure IE10 does not crash after 2nd search
		if(this.ua.indexOf("msie 10") != -1 && document.createEvent){
			function simulateEvent(target, evtType, options){
				var e = {
					bubbles: true, cancelable: (evtType != "mousemove"), view: window, detail: 0,
					screenX: 0, screenY: 0, clientX: 0, clientY: 0,
					ctrlKey: false, altKey: false, shiftKey: false, metaKey: false,
					button: 0, relatedTarget: undefined
				};
				StoreLocator.prototype.extend(e, options);
				var evt = document.createEvent("MouseEvents");
				evt.initMouseEvent(evtType, e.bubbles, e.cancelable, e.view, e.detail,
					e.screenX, e.screenY, e.clientX, e.clientY,
					e.ctrlKey, e.altKey, e.shiftKey, e.metaKey,
					e.button, e.relatedTarget || document.body.parentNode);
				target.dispatchEvent(evt);
				return evt;
			}
			var target = this.mapContainer;
			var x = this.container.offsetLeft + Math.floor(target.offsetWidth / 2);
			var y = this.container.offsetTop + Math.floor(target.offsetHeight / 2);
			var options = { clientX: x, clientY: y};
			simulateEvent(target, "mousedown", options);
			var options = { clientX: x + 1, clientY: y + 1};
			simulateEvent(document, "mousemove", options);
			var options = { clientX: x + target.offsetWidth, clientY: y + target.offsetHeight};
			simulateEvent(document, "mousemove", options);
			simulateEvent(document, "mousemove", options);
			simulateEvent(target, "mouseup", options);
		}
		this.showMap();
	},
	mapquestDirectionLink: function(address, data){
		return "http://www.mapquest.com/?le=t&q=" + address + "&maptype=map&vs=map";
	},
	googleDirectionLink: function(address, data){
		return "http://maps.google.com/?daddr=" + address;
	},
	leafletCleanUp: function(){
		this.mapContainer.style.display = "none";
		this.MAP.remove();
		this.MAP = null;
	},
	panEast: function(){
		var objId = StoreLocator.prototype.getObjectId(this);
		var slObj = objId == null ? this : StoreLocator.prototype.getTargetObject("uid", objId.split("Pan")[0]);
		slObj.MAP.panBy([slObj.panDistance, 0]);
	},
	panWest: function(){
		var objId = StoreLocator.prototype.getObjectId(this);
		var slObj = objId == null ? this : StoreLocator.prototype.getTargetObject("uid", objId.split("Pan")[0]);
		slObj.MAP.panBy([0 - slObj.panDistance, 0]);
	},
	panSouth: function(){
		var objId = StoreLocator.prototype.getObjectId(this);
		var slObj = objId == null ? this : StoreLocator.prototype.getTargetObject("uid", objId.split("Pan")[0]);
		slObj.MAP.panBy([0, slObj.panDistance]);
	},
	panNorth: function(){
		var objId = StoreLocator.prototype.getObjectId(this);
		var slObj = objId == null ? this : StoreLocator.prototype.getTargetObject("uid", objId.split("Pan")[0]);
		slObj.MAP.panBy([0, 0 - slObj.panDistance]);
	},
	zoomIn: function(){
		var objId = StoreLocator.prototype.getObjectId(this);
		var slObj = objId == null ? this : StoreLocator.prototype.getTargetObject("uid", objId.split("Zoom")[0]);
		slObj.MAP.zoomIn();
	},
	zoomOut: function(){
		var objId = StoreLocator.prototype.getObjectId(this);
		var slObj = objId == null ? this : StoreLocator.prototype.getTargetObject("uid", objId.split("Zoom")[0]);
		slObj.MAP.zoomOut();
	},
	leafletModuleLoaded: function(){
		return true;
	}
}, "StoreLocator.prototype.mapModuleLoaded", "leaflet");
